#include <iostream> // used for std::cout
#include <iomanip> // used for std::hex manipulation
#include <string> // used for std::string hexString
#include <sstream> // used for std::stringstream buffer
#include <algorithm> // used for std::transform
#include "O22SIOMM.h" // OptoMMP header
#include "O22STRCT.h" // OptoMMP header

int main() {
    const char *address = "127.0.0.1";
    // create a connection to the local EPIC, and confirm that it's successful
    O22SnapIoMemMap EPIC;
    int nResult = EPIC.OpenEnet2(address, 2001, 10000, 2, SIOMM_TCP);
    if(nResult != SIOMM_OK) {
        printf("OpenEnet2 reports result %d, exiting.\n", nResult);
        exit(0);
    }
    // wait for the connection to complete
    usleep(100000); // 0.1s

    // Use the `ReadBytes` function to get the byte array from a specific MMP address
//    ReadBytes(uint32_t dwOptoMMPAddress, int nLength, uint8_t * pbyary);
    uint32_t MMP = 0xF030002E; // address for the MAC address
    int length = 8; // MAC address is 64 bits = 8 bytes
    uint8_t response[8]; // variable to hold the eight bytes

    // Make the OptoMMP read command to get all 64 bits:
    nResult = EPIC.ReadBytes(MMP, length, response);
    
    // convert the 8 bytes into an unsigned, 2's compliment 64-bit integer
    uint64_t value = ((uint64_t)response[0] << 56 |
                (uint64_t)response[1] << 48 |
                (uint64_t)response[2] << 40 |
                (uint64_t)response[3] << 32 |
                (uint64_t)response[4] << 24 |
                (uint64_t)response[5] << 16 |
                (uint64_t)response[6] << 8 |
                (uint64_t)response[7]);
    // print the raw integer result (uncomment for debugging):
//    printf("MAC Address = %" PRIu64 "\n", value);
    
    std::stringstream buffer; // buffer to hold the number represented as a hexidecimal string
    // get the hex value of the unsigned 64-bit integer and save it to the buffer
    buffer << std::setw(16) << std::setfill('0') << std::hex << value;
    // save the string value of the buffer to a new string variable:
    std::string hexString = buffer.str();
    // the MAC address is only the first 12 letters:
    hexString = hexString.substr(0,12);
    
    /* Note: you can comment out or remove the following lines if you just need the raw value.
    */
    // First, apply `toupper` to every character in the hexidecimal string so it's all upper case:
    std::transform(hexString.begin(), hexString.end(), hexString.begin(), ::toupper);
    // insert 5 colon characters `:` to properly format the address:
    for(int i = 1; i < 6; i++) {
        hexString.insert((i*2+i)-1, 1, ':'); // you can replace ':' to separate with another character
    }
    //  `hexString` now holds a value in the form "AA:00:22:00:XX:YY"

    // print the result:
    std::cout << hexString << '\n';
    
    return 0;
}
